@@ -1,6 +1,6 @@
 /* 
    lock tests
-   Copyright (C) 2002-2005, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2002-2006, Joe Orton <joe@manyfish.co.uk>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -410,7 +410,7 @@ static int lock_compare(const char *ctx,
 }
 
 static void discover_result(void *userdata, const struct ne_lock *lk,
-			    const char *path, const ne_status *st)
+			    const ne_uri *uri, const ne_status *st)
 {
     struct result_args *args = userdata;
     args->result = lock_compare("discovered lock", lk, args->lock);
@@ -498,11 +498,11 @@ static int lock_shared(void)
 }
 
 static void dummy_discover(void *userdata, const struct ne_lock *lock,
-                           const char *uri, const ne_status *status)
+                           const ne_uri *uri, const ne_status *status)
 {
 }
 
-/* This failed with neon 0.23.x and earlier when memory leak detection
+/* This failed with neon 0.25.x and earlier when memory leak detection
  * is enabled. */
 static int fail_discover(void)
 {
@@ -537,16 +537,15 @@ static int fail_lockauth(void)
     ne_session *sess;
     struct ne_lock *lock;
     int ret;
+    struct many_serve_args args;
 
-    CALL(make_session(&sess, single_serve_string,
-                      "HTTP/1.1 401 Auth Denied\r\n"
-                      "WWW-Authenticate: Basic realm=\"realm@host\"\r\n"
-                      "Content-Length: 0\r\n"
-                      "\r\n"
-                      "HTTP/1.1 401 Auth Denied\r\n"
-                      "WWW-Authenticate: Basic realm=\"realm@host\"\r\n"
-                      "Connection: close\r\n"
-                      "\r\n"));
+    args.str = "HTTP/1.1 401 Auth Denied\r\n"
+        "WWW-Authenticate: Basic realm=\"realm@host\"\r\n"
+        "Content-Length: 0\r\n"
+        "\r\n";
+    args.count = 2;
+
+    CALL(make_session(&sess, many_serve_string, &args));
 
     ne_set_server_auth(sess, no_creds, NULL);
 
@@ -558,13 +557,13 @@ static int fail_lockauth(void)
     ne_lock_destroy(lock);
 
     lock = make_lock("/bar", "fish", ne_lockscope_exclusive, NE_DEPTH_ZERO);
+    lock->token = ne_strdup("opaquelocktoken:gah");
     ret  = ne_unlock(sess, lock);
     ONV(ret != NE_AUTH,
         ("attempt to unlock did not fail with NE_AUTH: %d (%s)",
          ret, ne_get_error(sess)));
     ne_lock_destroy(lock);
 
-
     CALL(await_server());
     ne_session_destroy(sess);
 
@@ -583,7 +582,7 @@ ne_test tests[] = {
     T(lock_timeout),
     T(lock_shared),
     T(discover),
-    T_XLEAKY(fail_discover),
+    T(fail_discover),
     T(fail_lockauth),
     T(NULL)
 };
