@@ -1,6 +1,6 @@
 /* 
    neon test suite
-   Copyright (C) 2002-2003, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2002-2006, Joe Orton <joe@manyfish.co.uk>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,22 +33,18 @@
 
 #include "ne_request.h"
 #include "ne_socket.h"
+#include "ne_ssl.h"
 #include "ne_auth.h"
 
 #include "tests.h"
 #include "child.h"
 #include "utils.h"
 
-#ifndef NEON_SSL
+#ifndef NE_HAVE_SSL
 /* this file shouldn't be built if SSL is not enabled. */
 #error SSL not supported
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include <openssl/evp.h>
-
-#define ERROR_SSL_STRING (ERR_reason_error_string(ERR_get_error()))
 
 #define SERVER_CERT "server.cert"
 #define CA_CERT "ca/cert.pem"
@@ -66,21 +62,10 @@ static ne_ssl_client_cert *def_cli_cert;
 static int check_dname(const ne_ssl_dname *dn, const char *expected,
                        const char *which);
 
-static int s_strwrite(SSL *s, const char *buf)
-{
-    size_t len = strlen(buf);
-    
-    ONV(SSL_write(s, buf, len) != (int)len,
-	("SSL_write failed: %s", ERROR_SSL_STRING));
-
-    return OK;
-}
-
 /* Arguments for running the SSL server */
 struct ssl_server_args {
     char *cert; /* the server cert to present. */
     const char *response; /* the response to send. */
-    int unclean; /* use an unclean shutdown if non-NULL */
     int numreqs; /* number of request/responses to handle over the SSL connection. */
 
     /* client cert handling: */
@@ -90,7 +75,10 @@ struct ssl_server_args {
     
     /* session caching: */
     int cache; /* use the session cache if non-zero */
-    SSL_SESSION *session; /* use to store copy of cached session. */
+    struct ssl_session {
+        unsigned char id[128];
+        size_t len;
+    } session;
     int count; /* internal use. */
 
     int use_ssl2; /* force use of SSLv2 only */
@@ -103,74 +91,43 @@ struct ssl_server_args {
 static int ssl_server(ne_socket *sock, void *userdata)
 {
     struct ssl_server_args *args = userdata;
-    int fd = ne_sock_fd(sock), ret;
-    /* we don't want OpenSSL to close this socket for us. */
-    BIO *bio = BIO_new_socket(fd, BIO_NOCLOSE);
+    int ret;
     char buf[BUFSIZ];
-    SSL *ssl;
-    static SSL_CTX *ctx = NULL;
+    static ne_ssl_context *ctx = NULL;
 
     if (ctx == NULL) {
-        ctx = SSL_CTX_new(args->use_ssl2 ? SSLv2_server_method()
-                          : SSLv23_server_method());
+        ctx = ne_ssl_context_create(args->use_ssl2 ? NE_SSL_CTX_SERVERv2
+                                    : NE_SSL_CTX_SERVER);
     }
 
-    ONV(ctx == NULL,
-	("could not create SSL_CTX: %s", ERROR_SSL_STRING));
-    ONV(!SSL_CTX_use_PrivateKey_file(ctx, server_key, SSL_FILETYPE_PEM),
-         ("failed to load private key: %s", ERROR_SSL_STRING));
+    ONV(ctx == NULL, ("could not create SSL context"));
+
+    ONV(ne_ssl_context_keypair(ctx, args->cert, server_key),
+        ("failed to load server keypair: ..."));
 
     NE_DEBUG(NE_DBG_HTTP, "using server cert %s\n", args->cert);
-    ONN("failed to load certificate",
-	!SSL_CTX_use_certificate_file(ctx, args->cert, SSL_FILETYPE_PEM));
 
-    if (args->require_cc) {
-        /* require a client cert. */
-        SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER | 
-                           SSL_VERIFY_FAIL_IF_NO_PEER_CERT, NULL);
- 
-        if (args->send_ca) {
-            /* the list of issuer DNs of these CAs is included in the
-            * certificate request message sent to the client */
-            SSL_CTX_set_client_CA_list(ctx, 
-                                       SSL_load_client_CA_file(args->send_ca));
-        }
-
-        /* set default ca_list */
-        if (!args->ca_list) args->ca_list = CA_CERT;
-    }
-    
-    if (args->ca_list) {
-        /* load the CA used to verify the client cert; also sent in the
-         * certificate exchange message. */
-        ONN("failed to load CA cert",
-            SSL_CTX_load_verify_locations(ctx, args->ca_list, NULL) != 1);
+    if (args->require_cc && !args->ca_list) {
+        args->ca_list = CA_CERT;
     }
 
-    if (args->cache && args->count == 0) {
-	/* enable OpenSSL's internal session cache, enabling the
-	 * negotiation to re-use a session if both sides support it. */
-	SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_SERVER);
-    }
+    ne_ssl_context_set_verify(ctx, args->require_cc, args->send_ca,
+                              args->ca_list);
 
-    ssl = SSL_new(ctx);
-    ONN("SSL_new failed", ssl == NULL);
+    ONV(ne_sock_accept_ssl(sock, ctx),
+        ("SSL accept failed: %s", ne_sock_error(sock)));
 
     args->count++;
 
-    SSL_set_bio(ssl, bio, bio);
-
-    ONV(SSL_accept(ssl) != 1,
-	("SSL_accept failed: %s", ERROR_SSL_STRING));
-
     /* loop handling requests: */
     do {
         const char *response = args->response ? args->response : DEF_RESP;
 
-        ret = SSL_read(ssl, buf, BUFSIZ - 1);
-        if (ret == 0)
+        ret = ne_sock_read(sock, buf, BUFSIZ - 1);
+        if (ret == NE_SOCK_CLOSED)
             return 0; /* connection closed by parent; give up. */
-        ONV(ret < 0, ("SSL_read failed (%d): %s", ret, ERROR_SSL_STRING));
+        ONV(ret < 0, ("SSL read failed (%d): %s", ret, 
+                      ne_sock_error(sock)));
         
         buf[ret] = '\0';
         
@@ -182,40 +139,40 @@ static int ssl_server(ne_socket *sock, void *userdata)
                 "Content-Length: 0\r\n" "\r\n";
         }
         
-        CALL(s_strwrite(ssl, response));
+        ONV(ne_sock_fullwrite(sock, response, strlen(response)),
+            ("SSL write failed: %s", ne_sock_error(sock)));
         
     } while (--args->numreqs > 0);
 
-    /* copy out the session if requested. */
-    if (args->session) {
-        SSL_SESSION *sess = SSL_get1_session(ssl);
 
-#if NE_DEBUGGING
-        /* dump session to child.log for debugging. */
-        SSL_SESSION_print_fp(ne_debug_stream, sess);
+    if (args->cache) {
+        unsigned char sessid[128];
+        size_t len = sizeof sessid;
+
+        ONN("could not retrieve session ID",
+            ne_sock_sessid(sock, sessid, &len));
+        
+#ifdef NE_DEBUGGING
+        {
+            char *b64 = ne_base64(sessid, len);
+            NE_DEBUG(NE_DBG_SSL, "Session id retrieved (%d): [%s]\n", 
+                     args->count, b64);
+            ne_free(b64);
+        }
 #endif
-
-        if (args->count == 0) {
-            /* save the session */
-            args->session = sess;
+        
+        if (args->count == 1) {
+            /* save the session. */
+            memcpy(args->session.id, sessid, len);
+            args->session.len = len;
         } else {
-            /* could just to do this with SSL_CTX_sess_hits really,
-             * but this is a more thorough test. */
-            ONN("cached SSL session not used",
-                SSL_SESSION_cmp(args->session, sess));
-            SSL_SESSION_free(args->session);
-            SSL_SESSION_free(sess);
+            /* Compare with stored session. */
+            ONN("cached session not used", 
+                args->session.len != len
+                || memcmp(args->session.id, sessid, len));
         }
     }	
     
-    if (!args->unclean) {
-	/* Erk, shutdown is messy! See Eric Rescorla's article:
-	 * http://www.linuxjournal.com/article.php?sid=4822 ; we'll just
-	 * hide our heads in the sand here. */
-	SSL_shutdown(ssl);
-	SSL_free(ssl);
-    }
-
     return 0;
 }
 
@@ -368,6 +325,13 @@ static int load_client_cert(void)
     ONV(name != NULL, ("noclient.p12 had friendly name `%s'", name));
     ne_ssl_clicert_free(cc);
 
+    /* test for ccert without a friendly name, noclient.p12 */
+    cc = ne_ssl_clicert_read("clientca.p12");
+    ONN("could not load clientca.p12", cc == NULL);
+    ONN("encrypted cert marked unencrypted?", !ne_ssl_clicert_encrypted(cc));
+    ONN("could not decrypt clientca.p12", ne_ssl_clicert_decrypt(cc, "foobar"));
+    ne_ssl_clicert_free(cc);
+
     /* tests for loading bogus files. */
     cc = ne_ssl_clicert_read("Makefile");
     ONN("loaded Makefile as client cert!?", cc != NULL);
@@ -408,6 +372,7 @@ static int simple_sslv2(void)
     ne_session *sess = ne_session_create("https", "localhost", 7777);
     struct ssl_server_args args = {SERVER_CERT, 0};
     args.use_ssl2 = 1;
+    ne_set_session_flag(sess, NE_SESSFLAG_SSLv2, 1);
     CALL(any_ssl_request(sess, ssl_server, &args, CA_CERT, NULL, NULL));
     ne_session_destroy(sess);
     return OK;
@@ -482,7 +447,7 @@ static int wildcard_init(void)
     struct stat stbuf;
     
     t_context("wildcard.cert not found:\n"
-	      "This test requires a Linux-like hostname command, see makekeys.sh");
+	      "Could not determine hostname/FQDN from makekeys.sh");
     PRECOND(stat("wildcard.cert", &stbuf) == 0);
 
     PRECOND(lookup_hostname() == OK);
@@ -725,10 +690,10 @@ static int get_failures(void *userdata, int fs, const ne_ssl_certificate *c)
 /* Helper function: run a request using the given self-signed server
  * certificate, and expect the request to fail with the given
  * verification failure flags. */
-static int fail_ssl_request(char *cert, char *cacert, 
+static int fail_ssl_request(char *cert, char *cacert, const char *host,
 			    const char *msg, int failures)
 {
-    ne_session *sess = DEFSESS;
+    ne_session *sess = ne_session_create("https", host, 7777);
     int gotf = 0, ret;
 
     ret = any_ssl_request(sess, fail_serve, cert, cacert,
@@ -746,21 +711,21 @@ static int fail_ssl_request(char *cert, char *cacert,
 	("verification flags were %d not %d", gotf, failures));
 
     /* and check that the request was failed too. */
-    ONN(msg, ret == NE_OK);
+    ONV(ret == NE_OK, ("%s", msg));
 
     ne_session_destroy(sess);
 
     return OK;
 }
 
-/* Note that the certs used for fail_* are all self-signed, so the
+/* Note that the certs used for fail_* are mostly self-signed, so the
  * cert is passed as CA cert and server cert to fail_ssl_request. */
 
 /* Check that a certificate with the incorrect commonName attribute is
  * flagged as such. */
 static int fail_wrongCN(void)
 {
-    return fail_ssl_request("wrongcn.pem", "wrongcn.pem",
+    return fail_ssl_request("wrongcn.pem", "wrongcn.pem", "localhost",
 			    "certificate with incorrect CN was accepted",
 			    NE_SSL_IDMISMATCH);
 }
@@ -769,8 +734,8 @@ static int fail_wrongCN(void)
 static int fail_expired(void)
 {
     char *c = ne_concat(srcdir, "/expired.pem", NULL);
-    CALL(fail_ssl_request(c, c, "expired certificate was accepted",
-                          NE_SSL_EXPIRED));
+    CALL(fail_ssl_request(c, c,  "localhost",
+                          "expired certificate was accepted", NE_SSL_EXPIRED));
     ne_free(c);
     return OK;
 }
@@ -778,7 +743,8 @@ static int fail_expired(void)
 static int fail_notvalid(void)
 {
     char *c = ne_concat(srcdir, "/notvalid.pem", NULL);
-    CALL(fail_ssl_request(c, c, "not yet valid certificate was accepted",
+    CALL(fail_ssl_request(c, c,  "localhost",
+                          "not yet valid certificate was accepted",
                           NE_SSL_NOTYETVALID));
     ne_free(c);
     return OK;    
@@ -788,14 +754,14 @@ static int fail_notvalid(void)
  * fail with UNTRUSTED. */
 static int fail_untrusted_ca(void)
 {
-    return fail_ssl_request("server.cert", NULL, "untrusted CA.",
-			    NE_SSL_UNTRUSTED);
+    return fail_ssl_request("server.cert", NULL, "localhost",
+                            "untrusted CA.", NE_SSL_UNTRUSTED);
 }
 
 static int fail_self_signed(void)
 {
-    return fail_ssl_request("ssigned.pem", NULL, "self-signed cert", 
-			    NE_SSL_UNTRUSTED);
+    return fail_ssl_request("ssigned.pem", NULL,  "localhost",
+                            "self-signed cert", NE_SSL_UNTRUSTED);
 }
 
 /* Test for failure when a server cert is presented which has no
@@ -815,6 +781,21 @@ static int fail_missing_CN(void)
     return OK;
 }                            
 
+/* test for a bad ipAddress altname */
+static int fail_bad_ipaltname(void)
+{
+    return fail_ssl_request("altname6.cert", CA_CERT, "127.0.0.1",
+                            "bad IP altname cert", NE_SSL_IDMISMATCH);
+}
+
+/* test for a ipAddress which matched against the hostname as per neon
+ * 0.24 behaviour. */
+static int fail_host_ipaltname(void)
+{
+    return fail_ssl_request("altname5.cert", CA_CERT, "localhost",
+                            "bad IP altname cert", NE_SSL_IDMISMATCH);
+}
+
 /* Test that the SSL session is cached across connections. */
 static int session_cache(void)
 {
@@ -957,6 +938,7 @@ static int ccert_unencrypted(void)
     args.require_cc = 1;
 
     ccert = ne_ssl_clicert_read("unclient.p12");
+    ONN("could not load unclient.p12", ccert == NULL);
     ONN("unclient.p12 was encrypted", ne_ssl_clicert_encrypted(ccert));
 
     ne_ssl_set_clicert(sess, ccert);
@@ -1171,6 +1153,7 @@ static int cert_identities(void)
     static const struct {
         const char *fname, *identity;
     } certs[] = {
+        { "ssigned.pem", "localhost" },
         { "twocn.cert", "localhost" },
         { "altname1.cert", "localhost" },
         { "altname2.cert", "nohost.example.com" },
@@ -1215,10 +1198,12 @@ static int check_validity(const char *fname,
 static int cert_validity(void)
 {
     char *cert = ne_concat(srcdir, "/expired.pem", NULL);
-    CALL(check_validity(cert, "Jan 21 20:39:04 2002 GMT", "Jan 31 20:39:04 2002 GMT"));
+    CALL(check_validity(cert, 
+                        "Mon, 21 Jan 2002 20:39:04 GMT", "Thu, 31 Jan 2002 20:39:04 GMT"));
     ne_free(cert);
     cert = ne_concat(srcdir, "/notvalid.pem", NULL);
-    CALL(check_validity(cert, "Dec 27 20:40:29 2023 GMT", "Dec 28 20:40:29 2023 GMT"));
+    CALL(check_validity(cert, 
+                        "Wed, 27 Dec 2023 20:40:29 GMT", "Thu, 28 Dec 2023 20:40:29 GMT"));
     ne_free(cert);
     return OK;
 }
@@ -1253,16 +1238,35 @@ static int dname_compare(void)
     return OK;
 }
 
+/* The dname with the UTF-8 encoding of the Unicode string: 
+ * "H<LATIN SMALL LETTER E WITH GRAVE>llo World". */
+#define I18N_DNAME "H\xc3\xa8llo World, Neon Hackers Ltd, Cambridge, Cambridgeshire, GB"
+
+/* N.B. t61subj.cert encodes an ISO-8859-1 string in a T61String
+ * field, which is strictly wrong but the common usage. */
+
 /* tests for ne_ssl_readable_dname */
 static int dname_readable(void)
 {
-    ne_ssl_certificate *cert;
+    struct {
+        const char *cert;
+        const char *subjdn, *issuerdn;
+    } ts[] = {
+        { "justmail.cert", "blah@example.com", NULL },
+        { "t61subj.cert", I18N_DNAME, NULL },
+        { "bmpsubj.cert", I18N_DNAME, NULL },
+        { "utf8subj.cert", I18N_DNAME, NULL },
+        { "twoou.cert", "First OU Dept, Second OU Dept, Neon Hackers Ltd, "
+          "Cambridge, Cambridgeshire, GB" }
+    };
+    size_t n;
 
-    cert = ne_ssl_cert_read("justmail.cert");
-    ONN("could not load justmail.cert", cert == NULL);
-
-    CALL(check_cert_dnames(cert, "blah@example.com", NULL));
-    ne_ssl_cert_free(cert);
+    for (n = 0; n < sizeof(ts)/sizeof(ts[0]); n++) {
+        ne_ssl_certificate *cert = ne_ssl_cert_read(ts[n].cert);
+        ONV(cert == NULL, ("could not load cert %s", ts[n].cert));
+        CALL(check_cert_dnames(cert, ts[n].subjdn, ts[n].issuerdn));
+        ne_ssl_cert_free(cert);
+    }
 
     return OK;
 }
@@ -1521,6 +1525,8 @@ ne_test tests[] = {
     T(fail_untrusted_ca),
     T(fail_self_signed),
     T(fail_missing_CN),
+    T(fail_host_ipaltname),
+    T(fail_bad_ipaltname),
 
     T(session_cache),
 	
